@@ -40,7 +40,7 @@
 #' sm_reg_metric(
 #'     metric_id = "Example",
 #'     entry = sm_new_metric(
-#'         fn = function(m) {
+#'         fn = function(m, ...) {
 #'             sm_area(sm_ytilde(m)) / 
 #'                 sm_area(sm_ref(m), order = sm_ytilde(m))
 #'         },
@@ -79,7 +79,11 @@ NULL
 .db_get <- function(key) {
     stopifnot(is.character(key))
     key <- key[[1]]
-    stopifnot(key %in% .db_list())
+    # TODO: compute levenshtein distance to suggest possible metrics
+    if (!key %in% .db_list()) {
+        stop("metric '", key, "' not found\n",
+             "use `sits_list_metrics()` to print metrics", call. = FALSE)
+    }
     
     get(key, envir = .db_env, inherits = FALSE)
 }
@@ -97,11 +101,14 @@ sm_new_metric <- function(fn,
                           fn_subset,
                           name = "", 
                           optimal = 0,
+                          summarizable = TRUE,
                           description = "", 
                           reference = "") {
     
     stopifnot(inherits(fn, "function"))
     stopifnot(is.character(name))
+    stopifnot(is.numeric(optimal))
+    stopifnot(is.logical(summarizable))
     stopifnot(is.character(description))
     stopifnot(is.character(reference))
     
@@ -110,6 +117,7 @@ sm_new_metric <- function(fn,
              fn_subset = fn_subset,
              name = name,
              optimal = optimal,
+             summarizable = summarizable,
              description = description,
              reference = reference),
         class = c("metric_entry", 
@@ -145,25 +153,23 @@ sm_unreg_metric <- function(metric_id) {
 #' @rdname db_functions
 sm_desc_metric <- function(metric_id) {
     
-    x <- .db_get(metric_id)
-    
-    if (nchar(x[["name"]]) > 0)
-        cat(paste("*", metric_id, paste0("(", x[["name"]], ")")), fill = TRUE)
-    else
-        cat(paste("*", metric_id), fill = TRUE)
-    
-    # print function body
-    cat(paste(" ", deparse(x[["fn"]])), fill = TRUE)
-    
-    if (nchar(x[["description"]]) > 0)
-        cat(paste(" ", x[["description"]]), fill = TRUE)
-    if (nchar(x[["reference"]]) > 0)
-        cat(paste(" ", "reference:", x[["reference"]]), fill = TRUE)
+    for (metric in metric_id) {
+        x <- .db_get(metric)
+        
+        if (nchar(x[["name"]]) > 0)
+            cat(paste("*", metric, paste0("(", x[["name"]], ")")), fill = TRUE)
+        else
+            cat(paste("*", metric), fill = TRUE)
+        
+        if (nchar(x[["description"]]) > 0)
+            cat(paste(" ", x[["description"]]), fill = TRUE)
+        if (nchar(x[["reference"]]) > 0)
+            cat(paste(" ", "reference:", x[["reference"]]), fill = TRUE)
+    }
 }
 
 .db_registry <- function() {
     
-    
     sm_reg_metric(
         metric_id = "OS2",
         entry = sm_new_metric(
@@ -171,6 +177,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_yprime,
             name         = "Oversegmentation",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Persello and Bruzzone (2010)"
         )
@@ -182,6 +189,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ystar,
             name         = "Oversegmentation",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Clinton et al. (2010)"
         )
@@ -193,6 +201,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_yprime,
             name         = "Undersegmentation",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Persello and Bruzzone (2010)"
         )
@@ -204,6 +213,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ystar,
             name         = "Undersegmentation",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Clinton et al. (2010)"
         )
@@ -215,7 +225,8 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_yprime,
             name         = "Area fit index",
             optimal      = 0,
-            description  = "Optimal value: 0",
+            summarizable = TRUE,
+            description  = "Optimal value is 0",
             reference    = "Lucieer and Stein (2002) and Clinton et al. (2010)"
         )
     )
@@ -226,6 +237,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ystar,
             name         = "Quality rate",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Weidner (2008) and Clinton et al. (2010)"
         )
@@ -237,6 +249,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ystar,
             name         = "Index D",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Levine and Nazif (1982) and Clinton et al. (2010)"
         )
@@ -248,6 +261,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_xprime,
             name         = "Precision",
             optimal      = 1,
+            summarizable = TRUE,
             description  = "Values from 0 to 1 (optimal)",
             reference    = "Van Rijsbergen (1979) and Zhang et al. (2015)"
         )
@@ -259,6 +273,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_yprime,
             name         = "Recall",
             optimal      = 1,
+            summarizable = TRUE,
             description  = "Values from 0 to 1 (optimal)",
             reference    = "Van Rijsbergen (1979) and Zhang et al. (2015)"
         )
@@ -270,6 +285,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ystar,
             name         = "underMerging",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 0.5",
             reference    = "Levine and Nazif (1982) and Clinton et al. (2010)"
         )
@@ -281,6 +297,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ystar,
             name         = "overMerging",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 0.5",
             reference    = "Levine and Nazif (1982) and Clinton et al. (2010)"
         )
@@ -292,6 +309,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_yprime,
             name         = "Match",
             optimal      = 1,
+            summarizable = TRUE,
             description  = "Values from 0 to 1 (optimal)",
             reference    = "Janssen and Molenaar (1995) and Feitosa et al. (2010)"
         )
@@ -305,6 +323,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_xprime,
             name         = "Evaluation measure",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 50",
             reference    = "Carleer et al. (2005)"
         )
@@ -316,8 +335,9 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ytilde,
             name         = "Relative area (RAsub)",
             optimal      = 1,
+            summarizable = FALSE,
             description  = "Values from 0 to 1 (optimal)",
-            reference    = "M\u00f6ller et al. (2007) and Clinton et al. (2010)"
+            reference    = "Moller et al. (2007) and Clinton et al. (2010)"
         )
     )
     sm_reg_metric(
@@ -327,8 +347,9 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ytilde,
             name         = "Relative area (RAsuper)",
             optimal      = 1,
+            summarizable = FALSE,
             description  = "Values from 0 to 1 (optimal)",
-            reference    = "M\u00f6ller et al. (2007) and Clinton et al. (2010)"
+            reference    = "Moller et al. (2007) and Clinton et al. (2010)"
         )
     )
     sm_reg_metric(
@@ -338,6 +359,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ytilde,
             name         = "Purity index",
             optimal      = 1,
+            summarizable = TRUE,
             description  = "Values from 0 to 1 (optimal)",
             reference    = "van Coillie et al. (2008)"
         )
@@ -349,7 +371,8 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_xprime,
             name         = "Fitness function",
             optimal      = 0,
-            description  = "Optimal value: 0",
+            summarizable = TRUE,
+            description  = "Optimal value is 0",
             reference    = "Costa et al. (2008)"
         )
     )
@@ -360,6 +383,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ycd,
             name         = "Oversegmentation",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Yang et al. (2014)"
         )
@@ -371,6 +395,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ycd,
             name         = "Undersegmentation",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Yang et al. (2014)"
         )
@@ -382,6 +407,7 @@ sm_desc_metric <- function(metric_id) {
             fn_subset    = sm_ycd,
             name         = "Euclidean distance",
             optimal      = 0,
+            summarizable = TRUE,
             description  = "Values from 0 (optimal) to 1",
             reference    = "Yang et al. (2014)"
         )
@@ -390,11 +416,96 @@ sm_desc_metric <- function(metric_id) {
         metric_id = "F_measure",
         entry = sm_new_metric(
             fn           = F_measure,
-            fn_subset    = sm_xprime,
+            fn_subset    = NULL,
             name         = "F-measure",
             optimal      = 1,
+            summarizable = TRUE,
             description  = "Values from 0 to 1 (optimal)",
             reference    = "Van Rijsbergen (1979) and Zhang et al. (2015)"
         )
     )
+    sm_reg_metric(
+        metric_id = "IoU",
+        entry = sm_new_metric(
+            fn           = IoU,
+            fn_subset    = sm_yprime,
+            name         = "Intersection over Union",
+            optimal      = 1,
+            summarizable = TRUE,
+            description  = "Values from 0 to 1 (optimal)",
+            reference = "Jaccard (1912); Rezatofighi et al. (2019)"
+        )
+    )
+    sm_reg_metric(
+        metric_id = "SimSize",
+        entry = sm_new_metric(
+            fn           = SimSize,
+            fn_subset    = sm_ystar,
+            name         = "Similarity of size",
+            optimal      = 1,
+            summarizable = TRUE,
+            description  = "Values from 0 to 1 (optimal)",
+            reference    = "Zhan et al. (2005)"
+        )
+    )
+    sm_reg_metric(
+        metric_id = "qLoc",
+        entry = sm_new_metric(
+            fn           = qLoc,
+            fn_subset    = sm_ystar,
+            name         = "Quality of object's location",
+            optimal      = 0,
+            summarizable = TRUE,
+            description  = "Optimal value is 0",
+            reference    = "Zhan et al. (2005)"
+        )
+    )
+    sm_reg_metric(
+        metric_id = "RPsub",
+        entry = sm_new_metric(
+            fn           = RPsub,
+            fn_subset    = sm_ytilde,
+            name         = "Relative position (sub)",
+            optimal      = 0,
+            summarizable = FALSE,
+            description  = "Optimal value is 0",
+            reference    = "Moller et al. (2007) and Clinton et al. (2010)"
+        )
+    )
+    sm_reg_metric(
+        metric_id = "RPsuper",
+        entry = sm_new_metric(
+            fn           = RPsuper,
+            fn_subset    = sm_ystar,
+            name         = "Relative position (super)",
+            optimal      = 0,
+            summarizable = FALSE,
+            description  = "Values from 0 (optimal) to 1",
+            reference    = "Moller et al. (2007) and Clinton et al. (2010)"
+        )
+    )
+    sm_reg_metric(
+        metric_id = "OI2",
+        entry = sm_new_metric(
+            fn           = OI2,
+            fn_subset    = sm_ytilde,
+            name         = "Overlap index",
+            optimal      = 0,
+            summarizable = FALSE,
+            description  = "Values from 0 to 1 (optimal)",
+            reference    = "Yang et al. (2017)"
+        )
+    )
+    sm_reg_metric(
+        metric_id = "Dice",
+        entry = sm_new_metric(
+            fn           = Dice,
+            fn_subset    = NULL,
+            name         = "Sorensen-Dice coefficient",
+            optimal      = 1,
+            summarizable = TRUE,
+            description  = "Values from 0 to 1 (optimal)",
+            reference    = "Dice (1945)"
+        )
+    )
 }
\ No newline at end of file
